/* MailFlag, a program to pulse some pins on a parallel port in order to raise
 * or lower a flag
 *
 * Written by Matt Mets in 2008
 * This code is released into the public domain
 *
 * Adapted from an example by Tomi Engdahl <then@delta.hut.fi>
 * http://www.epanorama.net/circuits/parallel_output.html
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/io.h>

#define base            0x378      /* base address of parallel port */

#define FLAG_DOWN       0x02
#define FLAG_UP         0x01

#define OK              0
#define BAD_DIRECTION   10
#define BAD_TIME        11
#define PORT_ERROR      12

/* Output the specified character for a certain amount of time
 *
 * address: address of the parallel port
 * direction: u for up, d for down.
 */
int doDirection(int address, char direction)
{
    unsigned char data;

    if(ioperm(address,1,1))
        return PORT_ERROR;
    
    switch(direction)
    {
        case 'u':
            data = FLAG_UP;
            break;
        case 'd':
            data = FLAG_DOWN;
            break;
        default:
            return BAD_DIRECTION;
    }

    outb(data, address);
    sleep(1);
    outb(0, address);

    return OK;
}


int main(int argc, char **argv) {
    int ret;

    if(argc != 2) {
        printf("%s: incorrect number of arguments specified\n", argv[0]);
        printf("Usage: %s direction\n", argv[0]);
        printf("Options:\n");
        printf("  direction: u for up or d for down\n");
        return BAD_DIRECTION;
    }

    ret = doDirection(base, *argv[1]);
    switch( ret ) {
        case OK:
            break;
        case BAD_DIRECTION:
            printf( "%s: Direction can be u or d\n", argv[0] );
            break;
        case PORT_ERROR:
            printf( "%s: Unable to open port for I/O (suid?)\n", argv[0] );
            break;
    }
    return ret;
}
