#!/usr/bin/python

# MailFlag, a program to pulse some pins on a parallel port in order to raise
# or lower a flag
#
# Written by Matt Mets in 2008
# This code is released into the public domain
#
# Adapted from an example by Tomi Engdahl <then@delta.hut.fi>
# http://www.epanorama.net/circuits/parallel_output.html
#

import socket, wx, os
import time

# Set these to corespond to your installation of biff
HOST = 'localhost'  # Host name (this should be fine)
PORT = 25501        # Port that biff is running on (check in Thunderbird)
POLL_TIME = 3       # How often the status should be checked (seconds)

# Expected values that the biff server will return
returnCodes = { '0' : 'NOTHING',
                '1' : 'NEW',
               '-1' : 'ERROR'}

# Location of the program to run to raise or lower the flag
FLAGPROGRAM = './domailflag'

# Try to connect and get a status update
def checkMail(host, port):
    try:
        mySock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        mySock.connect((host, port))
        mySock.settimeout(1)
        retval = mySock.recv(1)
        return returnCodes[retval]
    except:
        return 'ERROR'

# Support for a proper task bar icon
class MailFlagIcon(wx.TaskBarIcon):
    """ Task bar icon """
    def __init__(self,parent, host=HOST, port=PORT, poll_time=POLL_TIME):
        wx.TaskBarIcon.__init__(self)

        self.parentApp = parent
        self.host = host
        self.port = port
        self.icons = {
          'NOTHING':wx.Icon("icons/nothing.png",wx.BITMAP_TYPE_ANY),
          'NEW':wx.Icon("icons/new.png",wx.BITMAP_TYPE_ANY),
          'ERROR':wx.Icon("icons/error.png",wx.BITMAP_TYPE_ANY)}
        self.labels = {
          'NOTHING':'No new mail',
          'NEW':'New mail!!! Joy!!!',
          'ERROR':'Error communicating with Thunderbird'}

        self.CreateMenu()
        self.status = 'NOTHING'
        self.UpdateStatus(self.status)

        self.timer = wx.Timer(self, -1)
        self.Bind(wx.EVT_TIMER, self.OnTimer)
        self.timer.Start(1000*poll_time)

    def CreateMenu(self):
        self.Bind(wx.EVT_TASKBAR_RIGHT_UP,self.ShowMenu)
        self.Bind(wx.EVT_MENU,self.parentApp.ExitApp,id=wx.ID_EXIT)
        self.menu=wx.Menu()
        self.menu.Append(wx.ID_EXIT,"&Quit")

    def ShowMenu(self,e):
        self.PopupMenu(self.menu)

    def OnTimer(self, e):
        self.CheckMail()

    def CheckMail(self):
        newStatus = checkMail(self.host, self.port)
        if(self.status != newStatus):
           self.UpdateStatus(newStatus) 
        
    def UpdateStatus(self, newStatus):
        self.status = newStatus
        self.SetIcon(self.icons[self.status],self.labels[self.status])

        if(newStatus == 'NOTHING'):
            os.system(FLAGPROGRAM + ' d')
        if(newStatus == 'NEW'):
            os.system(FLAGPROGRAM + ' u')

#        print 'status updated: ' + newStatus

    def StopTimers(self):
        print 'no timers started'

class MailFlagMain(wx.Frame):
    """ Main window """
    def __init__(self,parent,id,title):
        wx.Frame.__init__(self,parent,id,title,size=(300,200))

        self.tbicon = MailFlagIcon(self)

    def ExitApp(self, e):
        self.tbicon.StopTimers()
        self.tbicon.RemoveIcon()
        self.tbicon.Destroy()
        self.Destroy()



app = wx.PySimpleApp()
frame = MailFlagMain(None, -1, 'Mail Flag')
app.MainLoop()
